'' =================================================================================================
''
''   File....... jm_rfid.spin
''   Purpose.... Object wrapper for Parallax serial RFID module
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 13 APR 2015
''
'' =================================================================================================


con { fixed io pins }

  RX1 = 31                                                       ' programming / terminal
  TX1 = 30

  SDA = 29                                                       ' eeprom / i2c
  SCL = 28

  
con

  ON_MS  = 1000                                                  ' reader activateion timing
  OFF_MS = 1000

  DISABLED = -1                                                  ' object state
  ENABLED  =  0
  READY    =  1


obj

  io   : "jm_io"                                                 '   basic io control
  time : "jm_time"                                               '   for timing and delays
  sin  : "jm_fullduplexserial"                                   ' * serial input from RFID reader


var

  long  cog                                                      ' cog running serial io

  long  enpin                                                    ' enable pin (to reader)

  long  state                                                    ' object state
  byte  tagbuf[11]                                               ' tag buffer 


pub start(en, rx)

'' Setup serial for RFID

  stop                                                           ' stop serial cog if running

  io.input(en)                                                   ' release pins in this cog
  io.input(rx)

  enpin := en                                                    ' copy enable pin for other methods
  
  cog := sin.start(rx, rx, %1100, 2400)                          ' start serial cog (1w, open-drain)

  if (cog)                                                       ' if serial available, enable 
    enable

  return cog


pub stop

'' Stops serial cog if running

  if (cog)
    cogstop(cog - 1)
    cog := 0

  disable                                                        ' disable RFID reader     
  bytefill(@tagbuf, 0, 11)                                       ' clear tag buffer


pub enable

'' Enable RFID system
'' -- call check() periodically after enabling for updated status

  ifnot (cog)                                                    ' serial must be running
    disable
    return

  io.low(enpin)                                                  ' RFID reader on
  sin.rxflush                                                    ' clean-up serial buffer
  time.start                                                     ' restart timer
  state := ENABLED


pub check | c, idx

'' Check status of RFID system
'' -- must be called before timer roll-over when enabled
'' -- returns current object state

  if (state == ENABLED)                                          ' if tag input enabled
    if (ina[enpin] == 0)                                         ' and reader active   
      c := sin.rxcheck                                           ' check for serial input
      if (c == $0A)                                              ' correct header?
        bytefill(@tagbuf, 0, 11)                                 ' yes, clear buffer
        repeat idx from 0 to 9                                   ' get 10 new tag bytes
          c := sin.rxtime(5)
          if (c => 0)
            tagbuf[idx] := c
        state := READY                                           ' mark ready
        io.input(enpin)                                          ' de-activate reader      

      else
        if (time.millis => ON_MS)                                ' on period expired?
          time.start                                             ' restart timer
          io.input(enpin)                                        ' de-activate reader 
  
    else { reader off }
      if (time.millis => OFF_MS)                                 ' off period expired?
        enable                                                   ' re-enable tag input
        
  return state


pub disable

'' Shuts down reader until enable() is called

  io.input(enpin)                                                ' RFID reader off
  state := DISABLED                                              ' do not reconnect via check()


pub status

  return state

  
pub address

'' Returns address of tagbuf
'' -- treated as 10-byte z-string

  return @tagbuf


pub tag_index(p_tags, tmax) | idx 

'' Returns index of current tag from list of tags
'' -- tags list must be stored as z-strings (10 ASCII bytes + 0)

  idx := 0

  repeat while (idx =< tmax)                                     ' iterate through list
    if (strcomp(@tagbuf, p_tags))                                ' if match
      return idx                                                 ' return index
    else
      ++idx                                                      ' next index
      p_tags += 11                                               ' skip over tag

  return -1                                                      ' not found in list


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}