'' =================================================================================================
''
''   File....... jm_rfid_demo__2015.spin
''   Purpose.... Identifies RFID tags while displaying a running timer
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started....
''   Updated.... 13 APR 2015
''
'' =================================================================================================


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                           ' use 5MHz crystal

  CLK_FREQ = (_clkmode >> 6) * _xinfreq                          ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                    ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                                ' ticks in 1us


  DISPLAY_MS = 4_000


con { io pins }

  RX1     = 31                                                   ' programming / terminal
  TX1     = 30

  SDA     = 29                                                   ' eeprom / i2c
  SCL     = 28

  RFID_RX = 17                                                   ' RFID on PAB
  RFID_EN = 16


obj

  io   : "jm_io"                                                 '   basic io control
  time : "jm_time"                                               '   timing and delays
  rfid : "jm_rfid"                                               ' * RFID input
  term : "jm_fullduplexserial"                                   ' * serial io for terminal
  str  : "jm_strings"                                            '   string methods


var

  long  showtag                                                  ' true when data data displayed


dat

  lastsecs    long      -1                                       ' for clock updates
  timestr     byte      "00:00:00", 0


pub main

  setup                                                          ' start program objects

  term.tx(term#CLS)

  repeat
    show_clock(0, 0)                                             ' show running clock
    check_tag


pub show_clock(x, y) | s, m, h

  s := time.seconds                                              ' get seconds from timer

  if (s == lastsecs)
    return

  lastsecs := s                                                  ' save for next check
  h := (s  / 3600) // 24                                         ' extract hours (keep 0..23)
  m := (s // 3600)  / 60                                         ' extract minutes
  s //= 60                                                       ' extract seconds

  timestr[0] := h  / 10 + "0"                                    ' format time
  timestr[1] := h // 10 + "0"
  timestr[3] := m  / 10 + "0"
  timestr[4] := m // 10 + "0"
  timestr[6] := s  / 10 + "0"
  timestr[7] := s // 10 + "0"

  cursor_xy(x, y)                                                ' move cursor
  term.str(@timestr)                                             ' print time


pub cursor_xy(x, y)

  term.tx(term#GOTOXY)
  term.tx(x)
  term.tx(y)


pub check_tag | state, tidx, t1  

  state := rfid.check                                            ' check on rfid tage
                                                                  
  ifnot (showtag)                                                ' if no tag showing
    if (state == rfid#READY)                                     ' has tag been presented?
      tidx := rfid.tag_index(@Tags, LAST_TAG)                    ' validate the tag
      if (tidx => 0)                                             ' if valid
        display_tag(tidx)                                        ' show the tag info
        showtag := true                                          ' enable timing of display
        time.start                                               ' restart timer (optional)
        t1 := time.millis                                        ' mark display sync point
        lastsecs := -1                                              
      else
        rfid.enable                                              ' re-enable after unknown tag

  else                                                            
    if ((time.millis - t1) => DISPLAY_MS)                        ' time to clear last tag?
      clear_tag                                                  ' clear previous tag info
      showtag := false                                           ' mark cleared
      rfid.enable                                                ' re-enable rfid

        
pub display_tag(tidx)

'' Displays tag information if tidx in valid range

  if ((tidx < 0) or (tidx > LAST_TAG))
    return

  cursor_xy(0, 2)

  term.str(string("Tag.... "))
  term.str(rfid.address)
  term.tx(term#CR)

  term.str(string("ID..... "))
  term.dec(tidx)
  term.tx(term#CR)

  term.str(string("Name... "))
  term.str(str.pntr(tidx, @Names))
  term.tx(term#CR)


pub clear_tag

'' Clears onscreen tag information

  cursor_xy(0, 2)
  term.tx(term#CLRDN)                                            ' clear screen from here
  showtag := false


pub setup

'' Setup IO and objects for application

  time.start                                                     ' start/sync time

  rfid.start(RFID_EN, RFID_RX)                                   ' start RFID

  term.start(RX1, TX1, %0000, 115_200)                           ' terminal via programming port


dat { tags data }

  Tags
  Tag0        byte      "0415148F26", 0
  Tag1        byte      "0415148E0C", 0
  Tag2        byte      "041514AEA3", 0
  Tag3        byte      "041514A076", 0
  Tag4        byte      "04129C1B43", 0


  LAST_TAG    byte      4


  Names
  Name0       byte      "Luke Skywalker", 0
  Name1       byte      "Princess Leia", 0
  Name2       byte      "Darth Vader", 0
  Name3       byte      "Obi Wan Kenobi", 0
  Name4       byte      "Yoda", 0


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}