'' =================================================================================================
''
''   File....... jm_rfid_entry.spin
''   Purpose....  
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2015 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 12 APR 2015 
''
'' =================================================================================================

{

  This program demonstrates reading tags and data from a text file
  -- file is ASCII text called "tags.txt"

  Tags are stored in file using this format (one definition per line):

  TAG 0123456789 Jon_McPhalen<CR>

  -- line must begin with "TAG"
  -- tag must be 10 ASCII characters
  -- names can be up to 20 characters
     * name MUST be included for each tag
     * underscore(s) in name will be replaced with space(s) for display

}


con { timing }

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000                                          ' use 5MHz crystal

  CLK_FREQ = _xinfreq * (_clkmode >> 6)                         ' system freq as a constant
  MS_001   = CLK_FREQ / 1_000                                   ' ticks in 1ms
  US_001   = CLK_FREQ / 1_000_000                               ' ticks in 1us 


  UNLOCK_MS = 2000
  

con { pab io pins }

  RX1     = 31                                                   ' programming / terminal
  TX1     = 30
  
  SDA     = 29                                                   ' eeprom / i2c
  SCL     = 28

  UNLOCK  = 27
  ER_LED  = 26
  
  SD_CS   = 25                                                   ' usd card
  SD_DI   = 24
  SD_CLK  = 23
  SD_DO   = 22

  ADC_CS  = 21                                                   ' adc input
  ADC_CLK = 20
  ADC_DO  = 19 { miso }
  ADC_DI  = 18 { mosi }
  
  RFID_RX = 17                                                   ' RFID on PAB
  RFID_EN = 16   
    

con 

  MAX_TAGS = 20                                                  ' make room for 20 tags

  #1, NO_MOUNT, NO_FILE, NO_TAGS                                 ' sd/file error codes

  COMMENT = 39                                                   ' apostrophe


obj

' main                                                           ' * master spin cog
  io     : "jm_io"                                               '   basic io
  time   : "jm_time"                                             '   delays/timing
  rfid   : "jm_rfid"                                             ' * RFID input
  term   : "jm_fullduplexserial"                                 ' * serial for terminal
  sd     : "fsrw"                                                ' * uSD file IO
  parser : "jm_parser"                                           '   text parser                                            
  str    : "jm_strings"                                          '   string methods
                                                                 
' * uses cog when loaded
  

var

  long  unlocked

  long  tagscount                                                ' tags loaded

  byte  linebuf[81]                                              ' line buffer for file read


dat { valid characters and tokens }

  CHAR_SET              byte    "$%+-_"    
                        byte    "0123456789"
                        byte    "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
                        byte    "abcdefghijklmnopqrstuvwxyz", 0
  
  TOKEN_LIST
  TOKEN_TAG             byte    "TAG", 0


con { enumerated token indexes }

  #0, T_TAG

  TOKEN_COUNT = T_TAG+1                                          ' total # of known tokens
  

dat

  TagsFile    byte      "TAGS.TXT", 0                            ' name must be DOS 8.3 format

  Tags        byte      0[MAX_TAGS * (10 + 1)]                   ' storage space for tags
  Names       byte      0[MAX_TAGS * (20 + 1)]                   ' storage space for names   


pub main | check, state, tidx, t1

  setup

  term.tx(term#CLS)
  term.str(string("Loading tags", 13))
  check := load_tags
  if (check)
    error(check)

  term.tx(term#CLS)
  show_tags
  repeat 2
    term.tx(term#CR)

  rfid.enable  

  repeat
    state := rfid.check                                          ' check on rfid tag 
    
    ifnot (unlocked)
      if (state == rfid#READY)                                   ' has tag been presented?
        term.str(rfid.address)
        repeat 2
          term.tx(" ")
        tidx := rfid.tag_index(@Tags, tagscount-1)               ' is this a known tag
        if (tidx => 0)
          unlocked := true
          io.high(UNLOCK)
          t1 := time.millis
          term.str(str.pntr(tidx, @Names))
        else
          term.str(string("Unknown tag"))
          rfid.enable
        term.tx(term#CR)            

    else
      if ((time.millis - t1) => UNLOCK_MS)                       ' time to clear last tag?
        unlocked := false
        io.low(UNLOCK)
        rfid.enable                                              ' re-enable rfid

    ' other code her


pub load_tags | len

  ifnot (mounted)
    return NO_MOUNT

  ifnot (open_file(@TagsFile, "r"))
    return NO_FILE

  ' sd mounted and tags file opened
  ' -- read and parse tags lines

  tagscount := 0                                                 ' no tags loaded

  repeat
    len := readln(@linebuf, 80)                                  ' read line from file
    if (len =< 0)                                                ' if end of file
      quit
    else
      str.trim(@linebuf)
      if (linebuf[0] <> COMMENT)
        process_line

  if (tagscount == 0)   
    return NO_TAGS
  else
    return 0 
    

pri process_line | check, p_token, p_dest

'' Should only be called from load_tags

  parser.reset                                                   ' ensure parser clear
  check := parser.enqueue_str(@linebuf)                          ' send line to parser
  if (check == false)                                            ' if line was bad                                           
    return                                                       '  abort

  if (parser.token_count <> 3)                                   ' check token count
    return

  p_token := parser.token_addr(0)                                ' point to 1st token 
  str.ucstr(p_token)                                             ' make uppercase                                           

  if (parser.get_token_id(p_token) <> T_TAG)                     ' correct token?
    return

  p_token := parser.token_addr(1)                                ' point to tag bytes        
  str.ucstr(p_token)                                             ' make uppercase
  if (strsize(p_token) <> 10)                                    ' if not correct length
    return                                                       '  abort

  p_dest := @Tags + (11 * tagscount)                             ' locate next space 
  bytemove(p_dest, p_token, 10)                                  ' save tag bytes
      
  p_token := parser.token_addr(2)                                ' point to name for tag
  p_dest := str.pntr(tagscount, @Names)                          ' locate place for next name
  bytemove(p_dest, p_token, strsize(p_token))                    ' copy name
  str.replace(p_dest, "_", " ")                                  ' replace underscore with space  

  ++tagscount


pub error(n)

  term.tx(term#CLS)

  case n
    NO_MOUNT:
      term.str(string(" Error: SD not mounted"))       
      
    NO_FILE:
      term.str(string(" Error: Cannot open tags file"))

    NO_TAGS:
      term.str(string(" Error: No tag definitions in file"))             

    other:
      term.str(string(" Unknown error..."))
      n := 4

  ' wait here for reboot

  repeat
    repeat n
      io.high(ER_LED)
      time.pause(75)
      io.low(ER_LED)
      time.pause(75)

    time.pause(1000 - (n * 150))


pub show_tags | tidx

'' Display tags that were loaded from SD card

  if (tagscount == 0)
    return

  tidx := 0

  repeat while (tidx < tagscount)
    term.rjdec(tidx, 2, " ")
    repeat 4
      term.tx(" ")
    term.str(str.pntr(tidx, @Tags)) 
    repeat 4
      term.tx(" ")
    term.str(str.pntr(tidx, @Names))   
    term.tx(term#CR)
    ++tidx

          
pub setup

  time.start                                                     ' start/sync time

  rfid.start(RFID_EN, RFID_RX)                                   ' start RFID
  rfid.disable 

  term.start(RX1, TX1, %0000, 115_200)                           ' terminal via programming port

  mount_sd                                                       ' start sd access

  parser.start(@CHAR_SET, @TOKEN_LIST, TOKEN_COUNT, true)        ' setup parsing engine    


con

  { ----------------- }
  {  F I L E   I / 0  } 
  { ----------------- }


var

  long  mounted
  

pub mount_sd | check

'' Attempt to mount SD card
'' -- returns true if SD card successfully mounted

  check := \sd.mount_explicit(SD_DO, SD_CLK, SD_DI, SD_CS)

  if (check => 0)
    mounted := true
  else
    mounted := false

  return mounted


pub dismount_sd

'' Unmounts SD card

  \sd.unmount
  mounted := false


pub open_file(p_str, mode) | check

'' Attempt to open file
'' -- p_str is pointer to filename (z-string)
'' -- mode is single character, (e.g., "r" for read -- must be lowercase)
'' -- sd card must be mounted before open

  if (mounted)
    check := \sd.popen(p_str, mode)                              ' attempt to open
    if (check => 0)                                               
      if (mode == "r")                        
        \sd.seek(0)                                              ' force sector read 
      return true
    else
      return false
  else
    return false


pub close_file

'' Closes open file

  \sd.pclose


pub readln(p_str, n) | c, len

'' Reads line of text from open text file
'' -- terminated by LF or EOF
'' -- p_str is address of string buffer
'' -- n is maximum number of characters to read from line

  len := 0                                                       ' index into string
                                                                  
  repeat n                                                        
    c := \sd.pgetc                                               ' get a character
    if (c < 0)                                                   ' end of file
      if (len == 0)                                              ' if nothing
        return -1                                                ' return empty
      else
        quit

    if (c <> 10)                                                 ' if not LF char
      byte[p_str++] := c                                         '  add to buffer
      len++                                                       
    else                                                         ' if LF
      quit                                                       '  we're done
                                                                  
  byte[p_str] := 0                                               ' terminate end of line
                                                                  
  return len
  

pub writeln(p_str, usecr, uself)

'' Writes z-string to open text file
'' -- set usecr to true to add CR
'' -- set uself to true to add LF

  \sd.pputs(p_str)
  if (usecr)
    \sd.pputc(13)
  if (uself)
    \sd.pputc(10)


pub read_buf(p_buf, n) | check

'' Reads n bytes from file to buffer at p_buf
'' -- returns number of bytes read

  return \sd.pread(p_buf, n)


pub write_buf(p_buf, n) | check   

'' Writes n bytes to file from buffer at p_buf
'' -- returns number of bytes written

  return \sd.pwrite(p_buf, n)


dat { license }

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}